"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const adminjs_1 = __importDefault(require("adminjs"));
const path_1 = require("path");
const path_to_regexp_1 = require("path-to-regexp");
const cookie_1 = __importDefault(require("cookie"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const compute_root_paths_1 = require("./utils/compute-root-paths");
const prepare_compare_path_1 = require("./utils/prepare-compare-path");
const routes_1 = require("./utils/routes");
const parse_files_1 = require("./utils/parse-files");
const DEFAULT_MAX_AGE = 900000;
exports.buildHandler = (adminOptions, options) => {
    let admin;
    let loginPath;
    let logoutPath;
    let rootPath;
    return async (req, res) => {
        if (!admin) {
            let beforeResult = null;
            if (options.before) {
                beforeResult = await options.before();
            }
            admin = new adminjs_1.default(beforeResult || adminOptions);
            ({ loginPath, logoutPath, rootPath } = admin.options);
            Object.assign(admin.options, compute_root_paths_1.computeRootPaths(admin.options, {
                project: process.env.GCLOUD_PROJECT,
                region: options.region,
                target: process.env.FUNCTION_TARGET,
                emulator: process.env.FUNCTIONS_EMULATOR,
            }, options.customFunctionPath));
        }
        const { method, query } = req;
        const path = prepare_compare_path_1.prepareComparePath(req.path, rootPath, options.customFunctionPath);
        const cookies = cookie_1.default.parse(req.headers.cookie || '');
        const token = cookies && cookies.__session;
        const currentAdmin = options.auth && token && token !== ''
            ? jsonwebtoken_1.default.verify(token, options.auth.secret)
            : null;
        if (options.auth) {
            const matchLogin = path_to_regexp_1.match(loginPath);
            if (matchLogin(path)) {
                if (method === 'GET') {
                    res.send(await admin.renderLogin({
                        action: admin.options.loginPath,
                        errorMessage: null,
                    }));
                }
                else {
                    const { email, password } = req.body;
                    const user = await options.auth.authenticate(email, password);
                    if (user) {
                        const session = jsonwebtoken_1.default.sign(user, options.auth.secret);
                        res.cookie('__session', session, {
                            maxAge: options.auth.maxAge || DEFAULT_MAX_AGE,
                        });
                        res.redirect(admin.options.rootPath);
                    }
                    else {
                        res.send(await admin.renderLogin({
                            action: admin.options.loginPath,
                            errorMessage: admin.translateMessage('invalidCredentials'),
                        }));
                    }
                }
                return;
            }
            const matchLogout = path_to_regexp_1.match(logoutPath);
            if (matchLogout(path)) {
                res.cookie('__session', '');
                res.redirect(admin.options.loginPath);
                return;
            }
            if (!currentAdmin) {
                res.redirect(admin.options.loginPath);
                return;
            }
            res.cookie('__session', token, {
                maxAge: options.auth.maxAge || DEFAULT_MAX_AGE,
            });
        }
        const route = routes_1.AppRoutes.find((r) => r.match(path) && r.method === method);
        if (route) {
            const params = route.match(path).params;
            const controller = new route.Controller({ admin }, currentAdmin);
            let fields = {};
            let files = {};
            if (method === 'POST') {
                ({ fields, files } = await parse_files_1.parseFiles(req));
            }
            const payload = Object.assign(Object.assign({}, fields), files);
            const html = await controller[route.action](Object.assign(Object.assign({}, req), { params, query, payload, method: method.toLowerCase() }), res);
            if (route.contentType) {
                res.set({ 'Content-Type': route.contentType });
            }
            if (html) {
                res.send(html);
            }
            if (files) {
                parse_files_1.cleanFiles(files);
            }
            return;
        }
        const asset = routes_1.AppAssets.find((r) => r.match(path));
        if (asset && !admin.options.assetsCDN) {
            res.status(200).sendFile(path_1.resolve(asset.src));
            return;
        }
        res.status(404).send('Page not found');
    };
};
